// Rendezvous: A Hopfield-Tank neural network implementation as a scheduler application
// Written by David J. Stein, Esq., 10/18/2004.  See "License.html" for terms and conditions of use.
//
// This module contains the code for adding or editing a meeting.

#region Using declarations
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
#endregion

namespace Hopfield_Tank_Neural_Network__Rendezvous_Namespace {

	public class AddEditMeeting : System.Windows.Forms.Form	{

		#region Variables and initialization code
		internal System.Windows.Forms.TextBox txtName;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Button btnCancel;
		private System.Windows.Forms.Button btnSave;
		private System.Windows.Forms.Label label1;
		internal System.Windows.Forms.ListBox lstAttendees;
		private System.ComponentModel.Container components = null;
  private ArrayList people;
		internal bool[] rsvp;

		public AddEditMeeting(ArrayList people)	{
			InitializeComponent();
			this.people = people;
			this.rsvp = new bool[people.Count];
			lstAttendees.BeginUpdate();
			lstAttendees.Items.Clear();
			for (int i = 0; i < people.Count; i++) {
				this.rsvp[i] = false;
				lstAttendees.Items.Add(((Person) people[i]).name);
			}
			lstAttendees.EndUpdate();
		}

		public AddEditMeeting(ArrayList people, bool[] rsvp, string name)	{
			InitializeComponent();
			txtName.Text = name;
			this.people = people;
			this.rsvp = new bool[people.Count];
			lstAttendees.BeginUpdate();
			lstAttendees.Items.Clear();
			for (int i = 0; i < people.Count; i++) {
				lstAttendees.Items.Add(((Person) people[i]).name);
				lstAttendees.SetSelected(i, rsvp[i]);
			}
			lstAttendees.EndUpdate();
		}

		#endregion

		#region Uninteresting Windows Form Designer generated code
		protected override void Dispose(bool disposing)		{
			if (disposing) {
				if (components != null) {
				components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

	private void InitializeComponent()	{
		this.txtName = new System.Windows.Forms.TextBox();
		this.label2 = new System.Windows.Forms.Label();
		this.btnCancel = new System.Windows.Forms.Button();
		this.btnSave = new System.Windows.Forms.Button();
		this.label1 = new System.Windows.Forms.Label();
		this.lstAttendees = new System.Windows.Forms.ListBox();
		this.SuspendLayout();
		// 
		// txtName
		// 
		this.txtName.Location = new System.Drawing.Point(48, 8);
		this.txtName.Name = "txtName";
		this.txtName.Size = new System.Drawing.Size(136, 20);
		this.txtName.TabIndex = 0;
		this.txtName.Text = "";
		this.txtName.KeyDown += new System.Windows.Forms.KeyEventHandler(this.txtName_KeyDown);
		// 
		// label2
		// 
		this.label2.Location = new System.Drawing.Point(8, 8);
		this.label2.Name = "label2";
		this.label2.Size = new System.Drawing.Size(40, 23);
		this.label2.TabIndex = 14;
		this.label2.Text = "Name";
		this.label2.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
		// 
		// btnCancel
		// 
		this.btnCancel.Location = new System.Drawing.Point(104, 208);
		this.btnCancel.Name = "btnCancel";
		this.btnCancel.Size = new System.Drawing.Size(80, 23);
		this.btnCancel.TabIndex = 2;
		this.btnCancel.Text = "Cancel";
		this.btnCancel.Click += new System.EventHandler(this.btnCancel_Click);
		// 
		// btnSave
		// 
		this.btnSave.Location = new System.Drawing.Point(8, 208);
		this.btnSave.Name = "btnSave";
		this.btnSave.Size = new System.Drawing.Size(80, 23);
		this.btnSave.TabIndex = 1;
		this.btnSave.Text = "Save";
		this.btnSave.Click += new System.EventHandler(this.btnSave_Click);
		// 
		// label1
		// 
		this.label1.Location = new System.Drawing.Point(8, 32);
		this.label1.Name = "label1";
		this.label1.Size = new System.Drawing.Size(72, 23);
		this.label1.TabIndex = 15;
		this.label1.Text = "Attendees";
		this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
		// 
		// lstAttendees
		// 
		this.lstAttendees.Location = new System.Drawing.Point(8, 56);
		this.lstAttendees.Name = "lstAttendees";
		this.lstAttendees.SelectionMode = System.Windows.Forms.SelectionMode.MultiSimple;
		this.lstAttendees.Size = new System.Drawing.Size(176, 147);
		this.lstAttendees.TabIndex = 16;
		this.lstAttendees.SelectedValueChanged += new System.EventHandler(this.lstAttendees_SelectedValueChanged);
		// 
		// AddEditMeeting
		// 
		this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
		this.ClientSize = new System.Drawing.Size(192, 240);
		this.Controls.Add(this.lstAttendees);
		this.Controls.Add(this.label1);
		this.Controls.Add(this.txtName);
		this.Controls.Add(this.label2);
		this.Controls.Add(this.btnCancel);
		this.Controls.Add(this.btnSave);
		this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
		this.MaximizeBox = false;
		this.MinimizeBox = false;
		this.Name = "AddEditMeeting";
		this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
		this.Text = "Meeting Details";
		this.ResumeLayout(false);

	}
		#endregion

		#region User interface event handlers
		private void btnSave_Click(object sender, System.EventArgs e) {
			DialogResult = (txtName.Text.Length > 0 ? DialogResult.OK : DialogResult.Cancel);
			Close();
		}

		private void btnCancel_Click(object sender, System.EventArgs e) {
			DialogResult = DialogResult.Cancel;
			Close();
		}

		private void txtName_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				DialogResult = (txtName.Text.Length > 0 ? DialogResult.OK : DialogResult.Cancel);
				Close();
			}
		}

		private void lstAttendees_SelectedValueChanged(object sender, System.EventArgs e) {
			for (int i = 0; i < lstAttendees.Items.Count; i++)
				rsvp[i] = lstAttendees.GetSelected(i);
		}
		#endregion
	}
}
