// Rendezvous: A Hopfield-Tank neural network implementation as a scheduler application
// Written by David J. Stein, Esq., 10/18/2004. See "License.html" for terms and conditions of use.
//
// This module contains all of the front-end code for the Rendezvous application.

#region Using declarations
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
#endregion

namespace Hopfield_Tank_Neural_Network__Rendezvous_Namespace {

	public class Rendezvous : System.Windows.Forms.Form	{

		#region Variables, initialization code, and Main
		private System.Windows.Forms.PictureBox picSchedule;
		private System.Windows.Forms.GroupBox groupBox2;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Label label4;
		private System.Windows.Forms.ListBox lstPeople;
		private System.Windows.Forms.ListBox lstMeetings;
		private System.Windows.Forms.ListBox lstRooms;
		private System.Windows.Forms.ListBox lstTimeSlots;
		private System.Windows.Forms.Button btnSolveStep;
		private System.Windows.Forms.Button btnSolveFull;
		private System.Windows.Forms.Button btnLoad;
		private System.Windows.Forms.Button btnSave;
		private System.Windows.Forms.Button btnAddPerson;
		private System.Windows.Forms.Button btnRemovePerson;
		private System.Windows.Forms.Button btnRemoveMeeting;
		private System.Windows.Forms.Button btnAddMeeting;
		private System.Windows.Forms.Button btnRemoveRoom;
		private System.Windows.Forms.Button btnAddRooms;
		private System.Windows.Forms.Button btnRemoveTime;
		private System.Windows.Forms.Button btnAddTime;
		private System.Windows.Forms.Button btnClear;
		private System.Windows.Forms.Button btnReset;
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.Panel panelSchedule;
		private System.Windows.Forms.GroupBox groupSchedule;
		private System.Windows.Forms.TextBox txtName;
		private System.Windows.Forms.TextBox txtStatus;
		private System.Windows.Forms.Label label5;

		private HopfieldTankNetwork network;

		[STAThread]
		static void Main() {
			Application.Run(new Rendezvous());
		}

		public Rendezvous() {
			InitializeComponent();
			network = new HopfieldTankNetwork((int) System.DateTime.Now.Ticks);
		}

		#endregion

		#region Uninteresting Windows Form Designer generated code

		protected override void Dispose(bool disposing) {
			if (disposing) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose(disposing);
		}

		private void InitializeComponent() {
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(Rendezvous));
			this.groupSchedule = new System.Windows.Forms.GroupBox();
			this.panelSchedule = new System.Windows.Forms.Panel();
			this.picSchedule = new System.Windows.Forms.PictureBox();
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.btnRemoveTime = new System.Windows.Forms.Button();
			this.btnAddTime = new System.Windows.Forms.Button();
			this.btnRemoveRoom = new System.Windows.Forms.Button();
			this.btnAddRooms = new System.Windows.Forms.Button();
			this.btnRemoveMeeting = new System.Windows.Forms.Button();
			this.btnAddMeeting = new System.Windows.Forms.Button();
			this.btnRemovePerson = new System.Windows.Forms.Button();
			this.btnAddPerson = new System.Windows.Forms.Button();
			this.lstTimeSlots = new System.Windows.Forms.ListBox();
			this.lstRooms = new System.Windows.Forms.ListBox();
			this.lstMeetings = new System.Windows.Forms.ListBox();
			this.lstPeople = new System.Windows.Forms.ListBox();
			this.label4 = new System.Windows.Forms.Label();
			this.label3 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.label1 = new System.Windows.Forms.Label();
			this.btnClear = new System.Windows.Forms.Button();
			this.btnSolveStep = new System.Windows.Forms.Button();
			this.btnSolveFull = new System.Windows.Forms.Button();
			this.btnReset = new System.Windows.Forms.Button();
			this.btnLoad = new System.Windows.Forms.Button();
			this.btnSave = new System.Windows.Forms.Button();
			this.txtName = new System.Windows.Forms.TextBox();
			this.txtStatus = new System.Windows.Forms.TextBox();
			this.label5 = new System.Windows.Forms.Label();
			this.groupSchedule.SuspendLayout();
			this.panelSchedule.SuspendLayout();
			this.groupBox2.SuspendLayout();
			this.SuspendLayout();
			// 
			// groupSchedule
			// 
			this.groupSchedule.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.groupSchedule.Controls.Add(this.panelSchedule);
			this.groupSchedule.Location = new System.Drawing.Point(8, 296);
			this.groupSchedule.Name = "groupSchedule";
			this.groupSchedule.Size = new System.Drawing.Size(712, 136);
			this.groupSchedule.TabIndex = 0;
			this.groupSchedule.TabStop = false;
			this.groupSchedule.Text = "Schedule";
			// 
			// panelSchedule
			// 
			this.panelSchedule.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.panelSchedule.AutoScroll = true;
			this.panelSchedule.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.panelSchedule.Controls.Add(this.picSchedule);
			this.panelSchedule.Location = new System.Drawing.Point(8, 16);
			this.panelSchedule.Name = "panelSchedule";
			this.panelSchedule.Size = new System.Drawing.Size(696, 112);
			this.panelSchedule.TabIndex = 0;
			// 
			// picSchedule
			// 
			this.picSchedule.Location = new System.Drawing.Point(0, 0);
			this.picSchedule.Name = "picSchedule";
			this.picSchedule.Size = new System.Drawing.Size(688, 104);
			this.picSchedule.TabIndex = 0;
			this.picSchedule.TabStop = false;
			this.picSchedule.Paint += new System.Windows.Forms.PaintEventHandler(this.picSchedule_Paint);
			// 
			// groupBox2
			// 
			this.groupBox2.Anchor = System.Windows.Forms.AnchorStyles.Top;
			this.groupBox2.Controls.Add(this.btnRemoveTime);
			this.groupBox2.Controls.Add(this.btnAddTime);
			this.groupBox2.Controls.Add(this.btnRemoveRoom);
			this.groupBox2.Controls.Add(this.btnAddRooms);
			this.groupBox2.Controls.Add(this.btnRemoveMeeting);
			this.groupBox2.Controls.Add(this.btnAddMeeting);
			this.groupBox2.Controls.Add(this.btnRemovePerson);
			this.groupBox2.Controls.Add(this.btnAddPerson);
			this.groupBox2.Controls.Add(this.lstTimeSlots);
			this.groupBox2.Controls.Add(this.lstRooms);
			this.groupBox2.Controls.Add(this.lstMeetings);
			this.groupBox2.Controls.Add(this.lstPeople);
			this.groupBox2.Controls.Add(this.label4);
			this.groupBox2.Controls.Add(this.label3);
			this.groupBox2.Controls.Add(this.label2);
			this.groupBox2.Controls.Add(this.label1);
			this.groupBox2.Location = new System.Drawing.Point(8, 72);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(712, 216);
			this.groupBox2.TabIndex = 1;
			this.groupBox2.TabStop = false;
			this.groupBox2.Text = "Resources";
			// 
			// btnRemoveTime
			// 
			this.btnRemoveTime.Location = new System.Drawing.Point(624, 184);
			this.btnRemoveTime.Name = "btnRemoveTime";
			this.btnRemoveTime.Size = new System.Drawing.Size(80, 23);
			this.btnRemoveTime.TabIndex = 7;
			this.btnRemoveTime.Text = "Remove";
			this.btnRemoveTime.Click += new System.EventHandler(this.btnRemoveTime_Click);
			// 
			// btnAddTime
			// 
			this.btnAddTime.Location = new System.Drawing.Point(536, 184);
			this.btnAddTime.Name = "btnAddTime";
			this.btnAddTime.Size = new System.Drawing.Size(80, 23);
			this.btnAddTime.TabIndex = 6;
			this.btnAddTime.Text = "Add";
			this.btnAddTime.Click += new System.EventHandler(this.btnAddTime_Click);
			// 
			// btnRemoveRoom
			// 
			this.btnRemoveRoom.Location = new System.Drawing.Point(448, 184);
			this.btnRemoveRoom.Name = "btnRemoveRoom";
			this.btnRemoveRoom.Size = new System.Drawing.Size(80, 23);
			this.btnRemoveRoom.TabIndex = 5;
			this.btnRemoveRoom.Text = "Remove";
			this.btnRemoveRoom.Click += new System.EventHandler(this.btnRemoveRoom_Click);
			// 
			// btnAddRooms
			// 
			this.btnAddRooms.Location = new System.Drawing.Point(360, 184);
			this.btnAddRooms.Name = "btnAddRooms";
			this.btnAddRooms.Size = new System.Drawing.Size(80, 23);
			this.btnAddRooms.TabIndex = 4;
			this.btnAddRooms.Text = "Add";
			this.btnAddRooms.Click += new System.EventHandler(this.btnAddRooms_Click);
			// 
			// btnRemoveMeeting
			// 
			this.btnRemoveMeeting.Location = new System.Drawing.Point(272, 184);
			this.btnRemoveMeeting.Name = "btnRemoveMeeting";
			this.btnRemoveMeeting.Size = new System.Drawing.Size(80, 23);
			this.btnRemoveMeeting.TabIndex = 3;
			this.btnRemoveMeeting.Text = "Remove";
			this.btnRemoveMeeting.Click += new System.EventHandler(this.btnRemoveMeeting_Click);
			// 
			// btnAddMeeting
			// 
			this.btnAddMeeting.Location = new System.Drawing.Point(184, 184);
			this.btnAddMeeting.Name = "btnAddMeeting";
			this.btnAddMeeting.Size = new System.Drawing.Size(80, 23);
			this.btnAddMeeting.TabIndex = 2;
			this.btnAddMeeting.Text = "Add";
			this.btnAddMeeting.Click += new System.EventHandler(this.btnAddMeeting_Click);
			// 
			// btnRemovePerson
			// 
			this.btnRemovePerson.Location = new System.Drawing.Point(96, 184);
			this.btnRemovePerson.Name = "btnRemovePerson";
			this.btnRemovePerson.Size = new System.Drawing.Size(80, 23);
			this.btnRemovePerson.TabIndex = 1;
			this.btnRemovePerson.Text = "Remove";
			this.btnRemovePerson.Click += new System.EventHandler(this.btnRemovePerson_Click);
			// 
			// btnAddPerson
			// 
			this.btnAddPerson.Location = new System.Drawing.Point(8, 184);
			this.btnAddPerson.Name = "btnAddPerson";
			this.btnAddPerson.Size = new System.Drawing.Size(80, 23);
			this.btnAddPerson.TabIndex = 0;
			this.btnAddPerson.Text = "Add";
			this.btnAddPerson.Click += new System.EventHandler(this.btnAddPerson_Click);
			// 
			// lstTimeSlots
			// 
			this.lstTimeSlots.HorizontalScrollbar = true;
			this.lstTimeSlots.Location = new System.Drawing.Point(536, 56);
			this.lstTimeSlots.Name = "lstTimeSlots";
			this.lstTimeSlots.Size = new System.Drawing.Size(168, 121);
			this.lstTimeSlots.TabIndex = 7;
			this.lstTimeSlots.DoubleClick += new System.EventHandler(this.lstTimeSlots_DoubleClick);
			// 
			// lstRooms
			// 
			this.lstRooms.HorizontalScrollbar = true;
			this.lstRooms.Location = new System.Drawing.Point(360, 56);
			this.lstRooms.Name = "lstRooms";
			this.lstRooms.Size = new System.Drawing.Size(168, 121);
			this.lstRooms.TabIndex = 6;
			this.lstRooms.DoubleClick += new System.EventHandler(this.lstRooms_DoubleClick);
			// 
			// lstMeetings
			// 
			this.lstMeetings.HorizontalScrollbar = true;
			this.lstMeetings.Location = new System.Drawing.Point(184, 56);
			this.lstMeetings.Name = "lstMeetings";
			this.lstMeetings.Size = new System.Drawing.Size(168, 121);
			this.lstMeetings.TabIndex = 5;
			this.lstMeetings.DoubleClick += new System.EventHandler(this.lstMeetings_DoubleClick);
			// 
			// lstPeople
			// 
			this.lstPeople.HorizontalScrollbar = true;
			this.lstPeople.Location = new System.Drawing.Point(8, 56);
			this.lstPeople.Name = "lstPeople";
			this.lstPeople.Size = new System.Drawing.Size(168, 121);
			this.lstPeople.TabIndex = 4;
			this.lstPeople.DoubleClick += new System.EventHandler(this.lstPeople_DoubleClick);
			this.lstPeople.SelectedIndexChanged += new System.EventHandler(this.lstPeople_SelectedIndexChanged);
			// 
			// label4
			// 
			this.label4.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.label4.Location = new System.Drawing.Point(536, 24);
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size(168, 23);
			this.label4.TabIndex = 3;
			this.label4.Text = "Time Slots";
			this.label4.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// label3
			// 
			this.label3.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.label3.Location = new System.Drawing.Point(8, 24);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(168, 23);
			this.label3.TabIndex = 2;
			this.label3.Text = "People";
			this.label3.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// label2
			// 
			this.label2.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.label2.Location = new System.Drawing.Point(360, 24);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(168, 23);
			this.label2.TabIndex = 1;
			this.label2.Text = "Rooms";
			this.label2.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// label1
			// 
			this.label1.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.label1.Location = new System.Drawing.Point(184, 24);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(168, 23);
			this.label1.TabIndex = 0;
			this.label1.Text = "Meetings";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// btnClear
			// 
			this.btnClear.Anchor = System.Windows.Forms.AnchorStyles.Top;
			this.btnClear.Location = new System.Drawing.Point(8, 40);
			this.btnClear.Name = "btnClear";
			this.btnClear.Size = new System.Drawing.Size(104, 23);
			this.btnClear.TabIndex = 0;
			this.btnClear.Text = "Clear Schedule";
			this.btnClear.Click += new System.EventHandler(this.btnClear_Click);
			// 
			// btnSolveStep
			// 
			this.btnSolveStep.Anchor = System.Windows.Forms.AnchorStyles.Top;
			this.btnSolveStep.Location = new System.Drawing.Point(232, 40);
			this.btnSolveStep.Name = "btnSolveStep";
			this.btnSolveStep.Size = new System.Drawing.Size(128, 23);
			this.btnSolveStep.TabIndex = 2;
			this.btnSolveStep.Text = "Solve Schedule (Step)";
			this.btnSolveStep.Click += new System.EventHandler(this.btnSolveStep_Click);
			// 
			// btnSolveFull
			// 
			this.btnSolveFull.Anchor = System.Windows.Forms.AnchorStyles.Top;
			this.btnSolveFull.Location = new System.Drawing.Point(368, 40);
			this.btnSolveFull.Name = "btnSolveFull";
			this.btnSolveFull.Size = new System.Drawing.Size(128, 23);
			this.btnSolveFull.TabIndex = 3;
			this.btnSolveFull.Text = "Solve Schedule (Full)";
			this.btnSolveFull.Click += new System.EventHandler(this.btnSolveFull_Click);
			// 
			// btnReset
			// 
			this.btnReset.Anchor = System.Windows.Forms.AnchorStyles.Top;
			this.btnReset.Location = new System.Drawing.Point(120, 40);
			this.btnReset.Name = "btnReset";
			this.btnReset.Size = new System.Drawing.Size(104, 23);
			this.btnReset.TabIndex = 1;
			this.btnReset.Text = "Reset Schedule";
			this.btnReset.Click += new System.EventHandler(this.btnReset_Click);
			// 
			// btnLoad
			// 
			this.btnLoad.Anchor = System.Windows.Forms.AnchorStyles.Top;
			this.btnLoad.Location = new System.Drawing.Point(616, 40);
			this.btnLoad.Name = "btnLoad";
			this.btnLoad.Size = new System.Drawing.Size(104, 23);
			this.btnLoad.TabIndex = 5;
			this.btnLoad.Text = "Load";
			this.btnLoad.Click += new System.EventHandler(this.btnLoad_Click);
			// 
			// btnSave
			// 
			this.btnSave.Anchor = System.Windows.Forms.AnchorStyles.Top;
			this.btnSave.Location = new System.Drawing.Point(504, 40);
			this.btnSave.Name = "btnSave";
			this.btnSave.Size = new System.Drawing.Size(104, 23);
			this.btnSave.TabIndex = 4;
			this.btnSave.Text = "Save";
			this.btnSave.Click += new System.EventHandler(this.btnSave_Click);
			// 
			// txtName
			// 
			this.txtName.Anchor = System.Windows.Forms.AnchorStyles.Top;
			this.txtName.Location = new System.Drawing.Point(104, 8);
			this.txtName.Name = "txtName";
			this.txtName.Size = new System.Drawing.Size(256, 20);
			this.txtName.TabIndex = 6;
			this.txtName.Text = "Sample Network";
			this.txtName.TextChanged += new System.EventHandler(this.txtName_TextChanged);
			// 
			// txtStatus
			// 
			this.txtStatus.Anchor = System.Windows.Forms.AnchorStyles.Top;
			this.txtStatus.Location = new System.Drawing.Point(368, 8);
			this.txtStatus.Name = "txtStatus";
			this.txtStatus.ReadOnly = true;
			this.txtStatus.Size = new System.Drawing.Size(352, 20);
			this.txtStatus.TabIndex = 7;
			this.txtStatus.Text = "Status: Empty network loaded.";
			// 
			// label5
			// 
			this.label5.Anchor = System.Windows.Forms.AnchorStyles.Top;
			this.label5.Location = new System.Drawing.Point(8, 8);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(88, 23);
			this.label5.TabIndex = 8;
			this.label5.Text = "Schedule Name";
			this.label5.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// Rendezvous
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(728, 438);
			this.Controls.Add(this.label5);
			this.Controls.Add(this.txtStatus);
			this.Controls.Add(this.txtName);
			this.Controls.Add(this.btnSave);
			this.Controls.Add(this.btnReset);
			this.Controls.Add(this.btnSolveFull);
			this.Controls.Add(this.btnSolveStep);
			this.Controls.Add(this.btnClear);
			this.Controls.Add(this.groupBox2);
			this.Controls.Add(this.groupSchedule);
			this.Controls.Add(this.btnLoad);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "Rendezvous";
			this.Text = "Rendezvous";
			this.Resize += new System.EventHandler(this.Rendezvous_Resize);
			this.Paint += new System.Windows.Forms.PaintEventHandler(this.Rendezvous_Paint);
			this.groupSchedule.ResumeLayout(false);
			this.panelSchedule.ResumeLayout(false);
			this.groupBox2.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		#region User interface event handlers

		private void btnAddPerson_Click(object sender, System.EventArgs e) {
			AddEditPerson p = new AddEditPerson();
			if (p.ShowDialog(this) == DialogResult.OK) {
				network.AddPerson(p.txtName.Text);
				UpdatePeopleList();
				ResizeSchedulePicture();
			}
		}

		private void lstPeople_DoubleClick(object sender, System.EventArgs e) {
			if (lstPeople.SelectedIndex != -1) {
				AddEditPerson p = new AddEditPerson(((Person) network.people[lstPeople.SelectedIndex]).name);
				if (p.ShowDialog(this) == DialogResult.OK) {
					((Person) network.people[lstPeople.SelectedIndex]).name = p.txtName.Text;
					UpdatePeopleList();
					ResizeSchedulePicture();
				}
			}
		}

		private void btnRemovePerson_Click(object sender, System.EventArgs e) {
			if (lstPeople.SelectedIndex != -1) {
				network.RemovePerson(lstPeople.SelectedIndex);
				UpdatePeopleList();
				UpdateMeetingsList();
				ResizeSchedulePicture();
			}
		}

		private void btnAddMeeting_Click(object sender, System.EventArgs e) {
			AddEditMeeting m = new AddEditMeeting(network.people);
			if (m.ShowDialog(this) == DialogResult.OK) {
				network.AddMeeting(m.txtName.Text);
				for (int i = 0; i < network.people.Count; i++)
					network.rsvp[network.meetings.Count - 1, i] = m.rsvp[i];
				UpdateMeetingsList();
				ResizeSchedulePicture();
			}
		}

		private void lstMeetings_DoubleClick(object sender, System.EventArgs e) {
			// construct one-dimensional array for this rsvp
			if (lstMeetings.SelectedIndex != -1) {
				bool[] rsvp = new bool[network.people.Count];
				for (int i = 0; i < network.people.Count; i++)
					rsvp[i] = network.rsvp[lstMeetings.SelectedIndex, i];
				if (lstMeetings.SelectedIndex != -1) {
					AddEditMeeting m = new AddEditMeeting(network.people, rsvp, ((Meeting) network.meetings[lstMeetings.SelectedIndex]).name);
					if (m.ShowDialog(this) == DialogResult.OK) {
						((Meeting) network.meetings[lstMeetings.SelectedIndex]).name = m.txtName.Text;
						for (int i = 0; i < network.people.Count; i++)
							network.rsvp[lstMeetings.SelectedIndex, i] = m.rsvp[i];
						UpdateMeetingsList();
						ResizeSchedulePicture();
					}
				}
			}
		}

		private void btnRemoveMeeting_Click(object sender, System.EventArgs e) {
			if (lstMeetings.SelectedIndex != -1) {
				network.RemoveMeeting(lstMeetings.SelectedIndex);
				UpdateMeetingsList();
				ResizeSchedulePicture();
			}
		}

		private void btnAddRooms_Click(object sender, System.EventArgs e) {
			AddEditRoom r = new AddEditRoom();
			if (r.ShowDialog(this) == DialogResult.OK) {
				network.AddRoom(r.txtName.Text, (int) r.nudCapacity.Value);
				UpdateRoomsList();
				ResizeSchedulePicture();
			}
		}

		private void lstRooms_DoubleClick(object sender, System.EventArgs e) {
			if (lstRooms.SelectedIndex != -1) {
				AddEditRoom r = new AddEditRoom(((Room) network.rooms[lstRooms.SelectedIndex]).name, ((Room) network.rooms[lstRooms.SelectedIndex]).capacity);
				if (r.ShowDialog(this) == DialogResult.OK) {
					((Room) network.rooms[lstRooms.SelectedIndex]).name = r.txtName.Text;
					((Room) network.rooms[lstRooms.SelectedIndex]).capacity = (int) r.nudCapacity.Value;
					UpdateRoomsList();
					ResizeSchedulePicture();
				}
			}
		}

		private void btnRemoveRoom_Click(object sender, System.EventArgs e) {
			if (lstRooms.SelectedIndex != -1) {
				network.RemoveRoom(lstRooms.SelectedIndex);
				UpdateRoomsList();
				ResizeSchedulePicture();
			}
		}

		private void btnAddTime_Click(object sender, System.EventArgs e) {
			AddEditTimeSlot t = new AddEditTimeSlot();
			if (t.ShowDialog(this) == DialogResult.OK) {
				network.AddTimeSlot(t.txtName.Text);
				UpdateTimeSlotsList();
				ResizeSchedulePicture();
			}
		}

		private void lstTimeSlots_DoubleClick(object sender, System.EventArgs e) {
			if (lstTimeSlots.SelectedIndex != -1) {
				AddEditTimeSlot t = new AddEditTimeSlot(((TimeSlot) network.timeslots[lstTimeSlots.SelectedIndex]).name);
				if (t.ShowDialog(this) == DialogResult.OK) {
					((TimeSlot) network.timeslots[lstTimeSlots.SelectedIndex]).name = t.txtName.Text;
					UpdateTimeSlotsList();
					ResizeSchedulePicture();
				}
			}
		}

		private void btnRemoveTime_Click(object sender, System.EventArgs e) {
			if (lstTimeSlots.SelectedIndex != -1) {
				network.RemoveTimeSlot(lstTimeSlots.SelectedIndex);
				UpdateTimeSlotsList();
				ResizeSchedulePicture();
			}
		}

		private void lstPeople_SelectedIndexChanged(object sender, System.EventArgs e) {
			btnRemovePerson.Enabled = (lstPeople.SelectedIndices.Count == 1);
		}

		private void picSchedule_Paint(object sender, System.Windows.Forms.PaintEventArgs e) {
			// setup drawing objects
			Font font = new Font("Times New Roman", 7.0f);
			StringFormat centerformat = new StringFormat();
			centerformat.Alignment = StringAlignment.Center;
			StringFormat leftformat = new StringFormat();
			leftformat.Alignment = StringAlignment.Center;
			Graphics g = e.Graphics;
			g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
			g.FillRectangle(System.Drawing.Brushes.BlanchedAlmond, e.ClipRectangle);
			if ((network.timeslots.Count >= 1) && (network.rooms.Count >= 1)) {
				// calculate row and column widths
				int headercolumnwidth = 0;
				foreach (object roomobject in network.rooms)
					headercolumnwidth = (int) Math.Max(headercolumnwidth, 8	+ g.MeasureString(((Room) roomobject).name, font).Width);
				int columnwidth = 0;
				foreach (object meetingobject in network.meetings)
					columnwidth = (int) Math.Max(columnwidth, g.MeasureString(((Meeting) meetingobject).name, font).Width + 7);
				foreach (object timeslotobject in network.timeslots)
					columnwidth = (int) Math.Max(columnwidth, g.MeasureString(((TimeSlot) timeslotobject).name, font).Width);
				int rowwidth = 24;

				// draw grid and enclosing box
				g.DrawRectangle(Pens.Black, 5, 5, headercolumnwidth + columnwidth * network.timeslots.Count - 5, rowwidth - 7 + network.rooms.Count * rowwidth);
				for (int i = 0; i < network.timeslots.Count; i++)
					g.DrawLine(Pens.Black, headercolumnwidth + i * columnwidth, rowwidth - 2, headercolumnwidth + i * columnwidth, rowwidth - 2 + network.rooms.Count * rowwidth);
				for (int i = 0; i < network.rooms.Count; i++)
					g.DrawLine(Pens.Black, headercolumnwidth, rowwidth - 2 + i * rowwidth, headercolumnwidth + columnwidth * network.timeslots.Count, rowwidth - 2 + i * rowwidth);

				// draw column headers (time slot names)
				for (int i = 0; i < network.timeslots.Count; i++)
					g.DrawString(((TimeSlot) network.timeslots[i]).name, font, Brushes.Black, headercolumnwidth + columnwidth * i + (columnwidth / 2), 9, centerformat);

				// draw row headers (room names)
				for (int i = 0; i < network.rooms.Count; i++)
					g.DrawString(((Room) network.rooms[i]).name, font, Brushes.Black, 5, rowwidth + rowwidth * i + 4);


				// draw row entries (conferences)
				for (int t = 0; t < network.timeslots.Count; t++) {
					for (int r = 0; r < network.rooms.Count; r++) {
						int index = -1;
						double best = 0.05;
						for (int m = 0; m < network.meetings.Count; m++) {
							if (network.V[t, r, m] > best) {
								best = network.V[t, r, m];
								index = m;
							}
						}
						if (index != -1) {  // draw this one
							Brush b;  // default (used for stuck items)
							if ((network.VPrevious[t, r, index] >= 0.995) && (network.V[t, r, index] >= 0.995))  // spot is locked near 1 - draw blue box
								b = new SolidBrush(Color.FromArgb(128, 128, 255));
							else if (Math.Abs(network.V[t, r, index] - network.VPrevious[t, r, index]) < 0.001)  // draw gray box
								b = new SolidBrush(Color.FromArgb((int) (network.V[t, r, index] * 255.0), 192, 192, 192));
							else if (network.V[t, r, index] >= network.VPrevious[t, r, index])  // spot is increasing - draw green box
								b = new SolidBrush(Color.FromArgb((int) (network.V[t, r, index] * 255.0), 128, 255, 128));
							else  // spot is declining - draw red box
								b = new SolidBrush(Color.FromArgb((int) (network.V[t, r, index] * 255.0), 255, 128, 128));
							g.DrawRectangle(Pens.Black, headercolumnwidth + t * columnwidth + 2, rowwidth + rowwidth * r, columnwidth - 4, rowwidth - 4);
							g.FillRectangle(b, headercolumnwidth + t * columnwidth + 3, rowwidth + rowwidth * r + 1, columnwidth - 6, rowwidth - 6);
							g.DrawString(((Meeting) network.meetings[index]).name, font, Brushes.Black, headercolumnwidth + t * columnwidth + (columnwidth / 2), rowwidth + rowwidth * r + 4, centerformat);
						}
					}
				}
			}
		}

		private void btnSave_Click(object sender, System.EventArgs e) {
			SaveFileDialog save = new SaveFileDialog();	
			save.Filter = "Rendezvous Network Files (*.net)|*.net";
			save.OverwritePrompt = true;
			if (save.ShowDialog() == DialogResult.OK) {
				network.Save(save.FileName);
				txtStatus.Text = "Status: Saved network.";
				txtStatus.Refresh();
			}
		}

		private void btnLoad_Click(object sender, System.EventArgs e) {
			OpenFileDialog open = new OpenFileDialog();
			open.Filter = "Rendezvous Network Files (*.net)|*.net";
			if (open.ShowDialog() == DialogResult.OK) {  // selected an XML file
				HopfieldTankNetwork newNetwork = HopfieldTankNetwork.Load(open.FileName);
				if (newNetwork != null) {
					network = newNetwork;
					txtName.Text = network.name;
					txtName.Refresh();
					txtStatus.Text = "Status: Loaded network.";
					txtStatus.Refresh();
					UpdatePeopleList();
					UpdateMeetingsList();
					UpdateRoomsList();
					UpdateTimeSlotsList();
					ResizeSchedulePicture();
				}
			}
		}

		private void btnClear_Click(object sender, System.EventArgs e) {
			network.Initialize();
			picSchedule.Refresh();
		}

		private void btnReset_Click(object sender, System.EventArgs e) {
			network.Reset();
			picSchedule.Refresh();
		}

		private void btnSolveStep_Click(object sender, System.EventArgs e) {
			network.Analyze();
			picSchedule.Refresh();
		}

		private void btnSolveFull_Click(object sender, System.EventArgs e) {
			EnableButtons(false);
			for (int i = 0; i < 1500; i++) {
				network.Analyze();
				picSchedule.Refresh();
				txtStatus.Text = "Status: Analyzing: Step " + i + "/1500   E=" + network.E;
				txtStatus.Refresh();
				if (network.E == 0.0)
					break;
			}
			EnableButtons(true);
		}

		private void txtName_TextChanged(object sender, System.EventArgs e) {
			network.name = txtName.Text;
		}

		#endregion

		#region Other event handlers

		private void Rendezvous_Resize(object sender, System.EventArgs e) {
			ResizeSchedulePicture();
		}

		private void Rendezvous_Paint(object sender, System.Windows.Forms.PaintEventArgs e) {
			picSchedule.Refresh();		
		}

		#endregion

		#region Helper functions

		private void UpdatePeopleList() {
			lstPeople.BeginUpdate();
			lstPeople.Items.Clear();
			foreach (object personobject in network.people) {
				Person person = (Person) personobject;
				lstPeople.Items.Add(person.name);
			}
			lstPeople.EndUpdate();
			lstPeople.Refresh();
		}

		private void UpdateMeetingsList() {
			lstMeetings.BeginUpdate();
			lstMeetings.Items.Clear();
			foreach (object meetingobject in network.meetings) {
				Meeting meeting = (Meeting) meetingobject;
				int count = 0;
				for (int i = 0; i < network.people.Count; i++)
					count += (network.rsvp[network.meetings.IndexOf(meeting), i] == true ? 1 : 0);
				lstMeetings.Items.Add(meeting.name + " (" + count + ")");
			}
			lstMeetings.EndUpdate();
			lstMeetings.Refresh();
		}

		private void UpdateRoomsList() {
			lstRooms.BeginUpdate();
			lstRooms.Items.Clear();
			foreach (object roomobject in network.rooms) {
				Room room = (Room) roomobject;
				lstRooms.Items.Add(room.name + " (" + room.capacity + ")");
			}
			lstRooms.EndUpdate();
			lstRooms.Refresh();
		}

		private void UpdateTimeSlotsList() {
			lstTimeSlots.BeginUpdate();
			lstTimeSlots.Items.Clear();
			foreach (object timeslotobject in network.timeslots) {
				TimeSlot timeslot = (TimeSlot) timeslotobject;
				lstTimeSlots.Items.Add(timeslot.name);
			}
			lstTimeSlots.EndUpdate();
			lstTimeSlots.Refresh();
		}

		private void ResizeSchedulePicture() {
			// This function sizes the picture according to the current size of the Rendezvous form. This is necessary becuase
			// the schedule picture object is housed in a panel in order to provide automatic scrollbar functionality.
			Font font = new Font("Times New Roman", 7.0f);
			Graphics g = Graphics.FromHwnd(this.Handle);
			// calculate row and column widths
			int headercolumnwidth = 0;
			foreach (object roomobject in network.rooms)
				headercolumnwidth = (int) Math.Max(headercolumnwidth, 8 + g.MeasureString(((Room) roomobject).name, font).Width);
			int columnwidth = 0;
			foreach (object meetingobject in network.meetings)
				columnwidth = (int) Math.Max(columnwidth, g.MeasureString(((Meeting) meetingobject).name, font).Width + 7);
			foreach (object timeslotobject in network.timeslots)
				columnwidth = (int) Math.Max(columnwidth, g.MeasureString(((TimeSlot) timeslotobject).name, font).Width);
			int rowwidth = 18;
			int scrollbarwidth = 18;
			picSchedule.Width = headercolumnwidth + columnwidth * network.timeslots.Count + 7;
			picSchedule.Height = 20 + rowwidth * network.rooms.Count + 12;
			picSchedule.Width = Math.Max(picSchedule.Width, panelSchedule.Width - 4);
			picSchedule.Height = Math.Max(picSchedule.Height, panelSchedule.Height - 4);
			if ((picSchedule.Height > panelSchedule.Height) && (picSchedule.Width == panelSchedule.Width - 4))
				picSchedule.Width -= scrollbarwidth;
			if ((picSchedule.Width > panelSchedule.Width) && (picSchedule.Height == panelSchedule.Height - 4))
				picSchedule.Height -= scrollbarwidth;

			this.panelSchedule.Refresh();
		}

		private void EnableButtons(bool enable) {
			btnAddMeeting.Enabled =
				btnAddPerson.Enabled =
				btnAddRooms.Enabled =
				btnAddTime.Enabled =
				btnClear.Enabled =
				btnLoad.Enabled =
				btnRemoveMeeting.Enabled =
				btnRemovePerson.Enabled =
				btnRemoveRoom.Enabled =
				btnRemoveTime.Enabled =
				btnReset.Enabled =
				btnSave.Enabled =
				btnSolveFull.Enabled =
				btnSolveStep.Enabled =
				lstMeetings.Enabled =
				lstPeople.Enabled =
				lstRooms.Enabled =
				lstTimeSlots.Enabled =
				enable;
		}

		#endregion

	}
}
